#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#include <math.h>

#include <hpic.h>

int main(int argc, char *argv[])
{
  char ch = '-';
  size_t i, j, k;

  char file[HPIC_STRNL];
  char outfile[HPIC_STRNL];
  double scale = 1.0;
  double offset = 0.0;

  size_t nside;
  int order;
  int coord;
  int type;

  char command[HPIC_STRNL];
  char comment[HPIC_STRNL];
  char creator[HPIC_STRNL];
  char extname[HPIC_STRNL];

  size_t nmaps;
  size_t mapnum = 0;
  hpic_int *pix = NULL;
  hpic_fltarr *maps = NULL;
  hpic_int *hits = NULL;
  hpic_float *errs = NULL;
  hpic_keys *keys = NULL;
  hpic_float *tempmap = NULL;

  int temp;

  if (argc == 1) {
    printf("\n");
    printf("<******************************************************>\n");
    printf("Usage:  hpic_scale\n");
    printf("        -in <input FITS file>\n");
    printf("        -out <output FITS file>\n");
    printf("        -map <map number to modify in file>\n");
    printf("        -scale <factor by which to multiply map>\n");
    printf("        -offset <offset to add to map>\n");
    printf("<******************************************************>\n\n");
    exit(1);
  }

  /* parse command line */

  for (i = 0; i < argc; i++) {
    if (argv[i][0] == ch) {
      if (!strcmp(argv[i], "-in")) {
        strncpy(file, argv[i + 1], HPIC_STRNL);
        i++;
      }
      if (!strcmp(argv[i], "-out")) {
        strncpy(outfile, argv[i + 1], HPIC_STRNL);
        i++;
      }
      if (!strcmp(argv[i], "-map")) {
        mapnum = (size_t) atoi(argv[i + 1]);
        i++;
      }
      if (!strcmp(argv[i], "-scale")) {
        scale = atof(argv[i + 1]);
        i++;
      }
      if (!strcmp(argv[i], "-offset")) {
        offset = atof(argv[i + 1]);
        i++;
      }
    }
  }

  /* read FITS file */

  if (hpic_fits_map_test(file, &nside, &order, &coord, &type, &nmaps)) {
    keys = hpic_keys_alloc();
    maps = hpic_fltarr_alloc(nmaps);
    for (i = 0; i < nmaps; i++) {
      tempmap = hpic_float_alloc(nside, order, coord, HPIC_AUTO);
      hpic_fltarr_set(maps, i, tempmap);
    }
    if (type == HPIC_FITS_FULL) {
      hpic_fits_full_read(file, creator, extname, maps, keys);
      tempmap = hpic_fltarr_get(maps, mapnum);
      hpic_float_scale(tempmap, scale);
      hpic_float_offset(tempmap, (float)offset);
      hpic_fits_full_write(outfile, creator, extname,
                           "Modified by hpic_scale", maps, keys);
    } else {
      errs = hpic_float_alloc(nside, order, coord, HPIC_AUTO);
      hits = hpic_int_alloc(nside, order, coord, HPIC_AUTO);
      pix = hpic_int_alloc(nside, order, coord, HPIC_AUTO);
      hpic_fits_cut_read(file, creator, extname, pix, hits, errs, maps, keys);
      tempmap = hpic_fltarr_get(maps, mapnum);
      hpic_float_scale(tempmap, scale);
      hpic_float_offset(tempmap, (float)offset);
      hpic_fits_cut_write(outfile, creator, extname, "Modified by hpic_scale",
                          pix, hits, errs, maps, keys);
      hpic_int_free(pix);
      hpic_int_free(hits);
      hpic_float_free(errs);
    }
    hpic_keys_free(keys);
    for (i = 0; i < nmaps; i++) {
      tempmap = hpic_fltarr_get(maps, i);
      hpic_float_free(tempmap);
    }
    hpic_fltarr_free(maps);
  } else {
    fprintf(stderr, "hpic_scale:  FITS file is not a recognized map type\n");
    exit(1);
  }

  return 0;
}
